<?php
/**
 * Clipsan Api Client Library
 *
 * @author ciwire
 * @package Clipsan Api
 */
class ClipsanApi
{

	/**
	 * project
	 *
	 * @var string
	 */
	protected $project;

	/**
	 * secureKey for project
	 *
	 * @var string
	 */
	protected $secureKey;

	/**
	 * Last call
	 *
	 * @var string
	 */
	public $lastcall;
	
	/**
	 * URL to Clipsan API
	 */
	const API_URL = "https://clipsan.com/api";

	/**
	 *
	 * @param string $project
	 * @param string $secureKey
	 *
	 */
	public function __construct($project, $secureKey)
	{
		$this->project = ($this->_testParam($project, "project", "string", true) ? $project : '');
		$this->secureKey = ($this->_testParam($secureKey, "secureKey", "string", true) ? $secureKey : '');
	}

	/**
     *
     * gets html code for player
     *
     * @param int $clipId
     * @param int $userId
     * @param string $userName
     * @param string $type
     */
	public function getPlayer($clipId, $userId = "", $userName = "", $type = "", $width = null, $height = null)
	{
		$params = '';
		$params .= $this->_makeParam(intval($clipId), "clipId", "int", true);
		$params .= $this->_makeParam(intval($userId), "userId", "int");
		$params .= $this->_makeParam($userName, "username", "string");
		$params .= $this->_makeParam($type, "type", "string");
		$params .= $this->_makeParam($width, "width", "int");
		$params .= $this->_makeParam($height, "height", "int");
		
		return $this->_makeRequest('/player', $params);
	}

	/**
     *
     * gets entry code for player
     *
     * @param int $clipId
     * @param int $userId
     * @param string $userName
     * @param string $type
     */
	public function getClipLink($clipId, $userId = "", $userName = "", $type = "")
	{
		$params = '';
		$params .= $this->_makeParam(intval($clipId), "clipId", "int", true);
		$params .= $this->_makeParam(intval($userId), "userId", "int");
		$params .= $this->_makeParam($userName, "username", "string");
		$params .= $this->_makeParam($type, "type", "string");
		
		$params .= $this->_createHashParam();
		
		$project = "/" . $this->project;
		
		return self::API_URL . '/link' . $project . $params;
	}

	/**
     *
     * gets movie download link
     *
     * @param int $clipId
     * @param int $userId
     * @param string $userName
     * @param string $email
     * @param string $ip
     * @param string $bitrate
     */
	public function getDownloadLink($clipId, $userId = "", $userName = "", $email = "", $ip = "", $bitrate = 0)
	{
		$params = '';
		$params .= $this->_makeParam($clipId, "clipId", is_int($clipId) ? 'int' : "string", true);
		$params .= $this->_makeParam(intval($userId), "userId", "int");
		$params .= $this->_makeParam($userName, "username", "string");
		$params .= $this->_makeParam($email, "email", "string");
		$params .= $this->_makeParam($ip ?: $_SERVER['REMOTE_ADDR'], "ip", "string");
		$params .= $this->_makeParam($bitrate == 'mp3' ? $bitrate : intval($bitrate), "bitrate", "string");
		
		return $this->_makeRequest('/download', $params);
	}

	/**
     *
     * gets entry code for player
     *
     * @param int $clipId
     * @param int $userId
     * @param string $userName
     * @param string $type
     */
	public function getTagJobLink($tag, $userId = "", $userName = "")
	{
		$params = '';
		$params .= $this->_makeParam($tag, "tag", "string", true);
		$params .= $this->_makeParam(intval($userId), "userId", "int");
		$params .= $this->_makeParam($userName, "username", "string");
		
		$params .= $this->_createHashParam();
		
		$project = "/" . $this->project;
		
		return self::API_URL . '/tagjob' . $project . $params;
	}

	/**
	 *
	 * gets html code for uploader
	 *
	 * @param int $userId
	 * @param string $userName
	 * @param int $type
	 * @param int $tag
	 */
	public function getUploader($userId = "", $userName = "", $type = "", $tag = "")
	{
		$params = '';
		$params .= $this->_makeParam(intval($userId), "userId", "int");
		$params .= $this->_makeParam($userName, "username", "string");
		$params .= $this->_makeParam($type, "type", "string");
		$params .= $this->_makeParam($tag, "tag", "string");
		
		return $this->_makeRequest('/uploader', $params);
	}

	/**
	 *
	 * gets clips list in xml, you can filter by method parameters
	 *
	 * @param int $userId
	 * @param string $username
	 * @param string $state
	 * @param string $tag
	 * @param int $circle
	 */
	public function getClipsList($userId = "", $username = "", $state = "", $tag = "", $circle = "")
	{
		$params = '';
		$params .= $this->_makeParam(intval($userId), "userId", "int");
		$params .= $this->_makeParam($username, "username", "string");
		$params .= $this->_makeParam($state, "state", "string");
		$params .= $this->_makeParam($tag, "tag", "string");
		$params .= $this->_makeParam($circle, "circleId", "int");
		
		return $this->_makeRequest('/list', $params);
	}

	/**
	 *
	 * gets statistics in xml format, which you can filter by method parameters
	 *
	 * @param int $userId
	 * @param string $userName
	 * @param int $clipId
	 */
	public function getStats($userId = "", $userName = "", $clipId = "")
	{
		$params = '';
		$params .= $this->_makeParam(intval($userId), "userId", "int");
		$params .= $this->_makeParam($userName, "userName", "string");
		$params .= $this->_makeParam(intval($clipId), "clipId", "int");
		
		return $this->_makeRequest('/stats', $params);
	}

	/**
	 * Get upload XML
	 *
	 */
	public function getUploadToken($xml)
	{
		return $this->_makeRequestPost('/uploadtoken', $xml);
	}

	protected function _makeRequestPost($resource, $xml)
	{
		$params .= $this->_createHashParam();
		$project = "/" . $this->project;
		
		$apiUrl = urldecode(self::API_URL . $resource . $project . $params);
		
		// init communication with API
		$channel = curl_init($apiUrl);
		
		curl_setopt($channel, CURLOPT_USERAGENT, $_SERVER['HTTP_USER_AGENT']);
		$data = array(
			'xml' => $xml);
		curl_setopt($channel, CURLOPT_POST, true);
		curl_setopt($channel, CURLOPT_POSTFIELDS, $data);
		
		curl_setopt($channel, CURLOPT_RETURNTRANSFER, TRUE);
		
		// get response
		$response = curl_exec($channel);
		$statusCode = curl_getinfo($channel, CURLINFO_HTTP_CODE);
		
		curl_close($channel);
		
		// test response
		$this->_testResponse($response, $statusCode);
		return $response;
	}

	/**
	 *
	 * This method is handling request to API server, getting and testing reponse
	 *
	 * @param string $resource
	 * @param string $params
	 */
	protected function _makeRequest($resource, $params)
	{
		$params .= $this->_createHashParam();
		
		$project = "/" . $this->project;
		
		$apiUrl = urldecode(self::API_URL . $resource . $project . $params);
		
		if (!function_exists('curl_init'))
			throw new RuntimeException("Missing CURL extension");
			
			// init communication with API
		$this->lastcall = $apiUrl;
		$channel = curl_init();
		curl_setopt($channel, CURLOPT_URL, $apiUrl);
		curl_setopt($channel, CURLOPT_USERAGENT, $_SERVER['HTTP_USER_AGENT']);
		curl_setopt($channel, CURLOPT_VERBOSE, TRUE);
		curl_setopt($channel, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($channel, CURLOPT_SSL_VERIFYHOST, FALSE);
		curl_setopt($channel, CURLOPT_RETURNTRANSFER, TRUE);
		
		// get response
		$response = curl_exec($channel);
		$statusCode = curl_getinfo($channel, CURLINFO_HTTP_CODE);
		
		curl_close($channel);
		
		// test response
		$this->_testResponse($response, $statusCode);
		return $response;
	}

	/**
	 * Create hash with time() + project + secureKey
	 *
	 * @return string
	 */
	protected function _createHashParam()
	{
		$time = time();
		$hash = md5($time . $this->secureKey . $this->project);
		return '/time/' . $time . '/hash/' . $hash;
	}

	/**
	 *
	 * This method tests reponse from server and return appropriate exception if reponse contains error message
	 *
	 * @param string $response
	 * @param int $statusCode
	 * @throws BadRequestException
	 * @throws UnauthorizedException
	 * @throws ForbiddenException
	 * @throws NotFoundException
	 * @throws InternalServerException
	 * @throws RuntimeException
	 */
	protected function _testResponse($response, $statusCode)
	{
		if (!$response)
		{
			throw new RuntimeException("Connecting to api on {$this->project} failed");
		}
		
		if ($statusCode != 200)
		{
			$resp = explode(";", $response);
			$code = intval($resp[0]);
			$message = $resp[1] . "({$resp[0]})";
			$message .= isset($resp[2]) ? ' - ' . $resp[2] : '';
			
			switch ($statusCode)
			{
				case "400":
					throw new BadRequestException($message, $code);
				break;
				case "401":
					throw new UnauthorizedException($message, $code);
				break;
				case "403":
					throw new ForbiddenException($message, $code);
				break;
				case "404":
					throw new NotFoundException($message, $code);
				break;
				case "500":
					throw new InternalServerException($message ? $message : "unknown error", $code);
				break;
				default:
					throw new RuntimeException($message ? $message : "unknown error");
				break;
			}
		}
	}

	/**
	 *
	 * This method is testing if param is ok and then building route part: /paramName/paramValue
	 *
	 * @param string $param
	 * @param string $name
	 * @param string $type
	 * @param bool $required
	 */
	protected function _makeParam($param, $name, $type, $required = false)
	{
		return ($this->_testParam($param, $name, $type, $required) ? "/$name/" . $param : '');
	}

	/**
	 *
	 * This method tests param to type and null/nut null and return appropriate exception if param is not valid
	 *
	 * @param string $param
	 * @param string $name
	 * @param string $type
	 * @param bool $required
	 * @throws InvalidArgumentException
	 */
	protected function _testParam($param, $name, $type, $required = false)
	{
		if ($param == null)
		{
			if ($required == true)
				throw new InvalidArgumentException("Request was not send - argument {$name}  is null", 400);
			
			return false;
		}
		
		switch ($type)
		{
			case "string":
				if (!is_string($param) && !is_int($param))
					throw new InvalidArgumentException("Request was not send - argument {$name}  is not {$type}", 400);
				;
			break;
			case "int":
				if (!is_int($param))
					throw new InvalidArgumentException("Request was not send - argument {$name}  is not {$type}", 400);
				;
			break;
		}
		
		return true;
	}
}

//exception definitions


/**
 * 400: Bad Request
 */
class BadRequestException extends LogicException
{}

/**
 * 401: Unauthorized
 */
class UnauthorizedException extends LogicException
{}

/**
 * 403: Forbidden
 */
class ForbiddenException extends LogicException
{}

/**
 * 404: Not Found
 */
class NotFoundException extends LogicException
{}

/**
 * 500: Internal Server Error
 */
class InternalServerException extends RuntimeException
{}
