<?php
/**
 * Clipsan plugin class
 *
 * @author CIWire
 */
class clipsan {
	/**
	 *
	 * @var string
	 */
	protected $server;
	
	/**
	 * Default constructor
	 */
	function clipsan() {
		register_activation_hook ( __FILE__, array (
				$this,
				'activation' 
		) );
		register_deactivation_hook ( __FILE__, array (
				$this,
				'deactivation' 
		) );
		
		add_action ( 'wp_head', array (
				$this,
				'frontHead' 
		) );
		add_action ( 'admin_menu', array (
				$this,
				'adminMenu' 
		) );
		
		$this->server = 'https://clipsan.com';
	}
	
	/**
	 * Get instance
	 *
	 * @return clipsan
	 */
	function &getInstance() {
		static $instance = null;
		
		if (! isset ( $instance )) {
			$instance = new clipsan ();
		}
		
		return $instance;
	}
	
	function activation() {
		add_option ( 'clipsan_url', 'https://www.clipsan.com/cs/movie/rss/' );
		add_option ( 'clipsan_userid' );
		add_option ( 'clipsan_securekey' );
		add_option ( 'clipsan_readall', false);
		add_option ( 'clipsan_secureall',  false );
		add_option ( 'clipsan_limitedplay', false );
		
		$table_name = $this->getPlayTableName ();
		
		$charset_collate = '';
		
		if (! empty ( $wpdb->charset ))
			$charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
		if (! empty ( $wpdb->collate ))
			$charset_collate .= " COLLATE $wpdb->collate";
		
		$sql = "CREATE TABLE $table_name (
			id int(11) NOT NULL AUTO_INCREMENT,
			hash varchar(20) NOT NULL,
			email varchar(100) DEFAULT NULL,
			ip varchar(255) DEFAULT NULL,
			added datetime NOT NULL,
		PRIMARY KEY (`id`),
		KEY `hash` (`hash`,`email`) USING BTREE
		) ENGINE=MyISAM $charset_collate";
		
		require_once (ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta ( $sql );
		
		$table_name = $this->getPassTableName ();
		$sql = "CREATE TABLE $table_name (
			id int(11) NOT NULL AUTO_INCREMENT,
	  		hash varchar(20) NOT NULL,
	  		email varchar(100) DEFAULT NULL,
	  		password varchar(255) DEFAULT NULL,
	  		added datetime NOT NULL,
  		PRIMARY KEY (`id`),
  		KEY `hash` (`email`,`hash`) USING BTREE
		) ENGINE=MyISAM $charset_collate";
		
		dbDelta ( $sql );
	}
	
	function deactivation() {
		global $wpdb;
		
		delete_option ( 'clipsan_url' );
		delete_option ( 'clipsan_userid' );
		delete_option ( 'clipsan_securekey' );
		delete_option ( 'clipsan_readall' );
		delete_option ( 'clipsan_secureall' );
		delete_option ( 'clipsan_limitedplay' );
		
		$sql = "DROP TABLE IF EXISTS " . $this->getPlayTableName () . ";";
		$wpdb->query ( $sql );
		
		$sql = "DROP TABLE IF EXISTS " . $this->getPassTableName () . ";";
		$wpdb->query ( $sql );
	}
	
	function frontHead() {
		// printf('<link rel="stylesheet" type="text/css" href="%s" />',
		// plugins_url('clipsan-video/clipsan.css'));
	}
	
	function adminMenu() {
		add_options_page ( 'Clipsan videa', 'Clipsan videa', 7, __FILE__, array (
				$this,
				'admin' 
		) );
	}
	
	public function readVideos($extended = false, $tag = '') {
		// init
		$clipsan_url = get_option ( 'clipsan_url' );
		$clipsan_userid = get_option ( 'clipsan_userid' );
		$clipsan_securekey = get_option ( 'clipsan_securekey' );
		$items = array ();
		
		// setup url
		require_once 'ClipsanApi.php';
		$clipsan = new ClipsanApi ( 'clipsan', $clipsan_securekey );
		
		try {
			$result = $clipsan->getClipsList ( $clipsan_userid, "", "", $tag );
			return simplexml_load_string ( $result );
		} catch ( RuntimeException $e ) {
			echo 'Error: ' . $e->getMessage ();
			return;
		}
	}
	
	public function getVideos($width = 680, $height = 382) {
		$videos = array ();
		foreach ( $this->readVideos ( true ) as $videoXml ) {
			
			$video = simplexml_load_string ( $videoXml );
			$hash = substr ( ( string ) $video->link, strrpos ( ( string ) $video->link, '/' ) + 1 );
			$object = new stdClass ();
			$object->hash = $hash;
			$object->title = trim ( ( string ) $video->title );
			$data = json_decode ( $video->description );
			$object->desc = trim ( $data->description );
			$object->cast = trim ( $data->cast );
			$object->duration = intval ( $data->duration );
			$object->image = "http://clipsan.com/cs/movie/player/preview/hash/$hash/w/$width/h/$height";
			$object->category = array ();
			foreach ( $video->category as $key => $category ) {
				$object->category [] = ( string ) $category;
			}
			$videos [$hash] = $object;
		}
		return $videos;
	}
	
	function listing($atts) {
		extract ( shortcode_atts ( array (
				'tag' => '',
				'lang' => 'cs' 
		), $atts ) );
		
		ini_set('display_errors', 'on');
		error_reporting(E_ALL);
		
		// iterate items
		$items = array();
		foreach ( $this->readVideos ( false, $tag ) as $item ) {
			// display item
			foreach ( $item->translation as $translation ) {
				if ((string)$translation->attributes ()->lang == $lang)
				{
					break;
				}
			}
			if (isset($translation))
			{
				$title = (string)$translation->attributes ()->title;
				if (key_exists($title, $items))
				{
					$items[$title.'-'.rand(1000, 9999)] = $item;
				}
				else
				{
					$items[$title] = $item;
				}
			}
		}
		ksort($items);
		foreach ($items as $item)
		{
			$this->displayVideo ( $item );
		}		
	}
	
	public function instantView($lang, $hash, $width, $height, $type = null) {
		global $current_user;
		get_currentuserinfo ();
		
		// fetch xml
		$url = "{$this->server}/$lang/player/hash/$hash/width/$width/height/$height/time/" . time () . "/autoplay/0/ip/" . $_SERVER ['REMOTE_ADDR'];
		if ($current_user->user_email) {
			$url .= '/custom/' . $current_user->user_email;
		}
		if ($type) {
			$url .= '/type/' . $type;
		}
		$url .= "?location=" . urlencode ( $this->url () );
		$url .= "&HTTP_USER_AGENT=" . urlencode ( $_SERVER ['HTTP_USER_AGENT'] );
		$json = @file_get_contents ( $url );
		$array = json_decode ( substr ( $json, 1, strlen ( $json ) - 2 ), true );
		
		if (is_array ( $array ) && key_exists ( 'html', $array )) {
			return '<script type="text/javascript" src="http://static.clipsan.com/js/jwplayer.js"></script><div class="clipsan-player">' . $array ['html'] . '</div>';
		}
		return false;
	}
	
	public function readVideo($hash) {
		$url = "{$this->server}/api/clip/clipsan/?hash=$hash";
		$json = file_get_contents ( $url );
		return json_decode ( $json, true );
	}
	
	protected function url() {
		return ($_SERVER ['SERVER_PORT'] == 80 ? 'http://' : 'https://') . $_SERVER ['SERVER_NAME'] . $_SERVER ['REQUEST_URI'];
	}
	
	/**
	 * Fetch XML tag value
	 *
	 * @param string $xml        	
	 * @param string $tag        	
	 * @return string
	 */
	function _fetchXmlTag($xml, $tag) {
		$matches = array ();
		
		preg_match ( "/\<{$tag}\>(\<!\[CDATA\[)?(.*?)(\]\]\>)?\<\/{$tag}\>/i", $xml, $matches );
		
		return isset ( $matches [2] ) ? $matches [2] : null;
	}
	
	/**
	 * Fetch XML images
	 *
	 * @param string $xml        	
	 * @return array
	 */
	function _fetchXmlImages($xml) {
		$matches = array ();
		
		preg_match_all ( "/\<enclosure.+type=\"image.+\".+url=\"(.*?)\"/i", $xml, $matches );
		
		return $matches [1];
	}
	
	function displayVideo(SimpleXMLElement $xml) {
		global $current_user;
		$hash = ( string ) $xml->attributes ()->movieHash;
		$movieWidth = ( int ) $xml->attributes ()->width;
		$movieHeight = ( int ) $xml->attributes ()->height;
		$unique = $xml->attributes ()->unique;
		$lang = 'cs';
		$width = 960;
		$height = ceil ( $width / ($movieWidth / $movieHeight) );
		$image = $this->_fetchXmlImages ( $xml );
		$image = array_shift ( $image );
		
		foreach ( $xml->translation as $translation ) {
			if ((string)$translation->attributes ()->lang == $lang)
			{
				break;
			}
		}
		if (isset($translation))
		{
		wp_enqueue_script ( 'clipsan', "//static.clipsan.com/js/player5.js", array (
		'jquery'
		) );
		$title = $translation->attributes ()->title;
		echo '
		<div class="clipsan-video">
		<div class="env">
		'.($title ? '<h2>' .$title . '</h2>' : '').'
		<div class="clipsan-player" data-movie="' . $hash . '" data-width="' . $width . '" data-height="' . $height . '" data-lang="' . $lang . '" data-unique="' . $unique . '" data-scale="1" data-custom="' . $current_user->user_email . '" ></div>
		'.($translation->description ? '<p>' . nl2br($translation->description) . '</p>' : '').'
		</div>
		</div>';
		}
	
	}
	
	function admin() {
		$clipsan_securekey = get_option ( 'clipsan_securekey' );
		$clipsan_readall = get_option ( 'clipsan_readall' );
		$clipsan_secureall = get_option ( 'clipsan_secureall' );
		$clipsan_limitedplay = get_option ( 'clipsan_limitedplay' );
		
		if (isset ( $_POST ['clipsan_submit'] )) {
			$clipsan_securekey = stripslashes ( $_POST ['clipsan_securekey'] );
			$clipsan_readall = intval ( $_POST ['clipsan_readall'] ) == 1;
			$clipsan_secureall = intval ( $_POST ['clipsan_secureall'] ) == 1;
			$clipsan_limitedplay = intval ( $_POST ['clipsan_limitedplay'] ) == 1;
			
			update_option ( 'clipsan_securekey', $clipsan_securekey );
			update_option ( 'clipsan_readall', $clipsan_readall );
			update_option ( 'clipsan_secureall', $clipsan_secureall );
			update_option ( 'clipsan_limitedplay', $clipsan_limitedplay );
			
			if (intval ( $_POST ['clipsan_per'] ) && intval ( $_POST ['clipsan_per_stage'] ))
			{
				update_option ( 'clipsan_per_'.intval ( $_POST ['clipsan_per'] ), intval ( $_POST ['clipsan_per_stage'] ) );
			}
		}
		
		?>
<div class="wrap">
	<h2><?php echo esc_html( 'Nastavení pro videa z Clipsanu' ); ?></h2>
	<form name="form_mt" method="post" action="">
		<table class="form-table">
			<tr valign="top">
				<th scope="row"><label for="clipsan_securekey"><?php _e('Chráněný klíč', 'clipsan-video') ?></label></th>
				<td><input name="clipsan_securekey" type="text"
					value="<?php echo $clipsan_securekey; ?>" id="clipsan_securekey"
					class="regular-text" placeholder="<?php _e('Zadejte chráněný klíč', 'clipsan-video') ?>"
					required="required" /><br /><span class="description"><?php _e('Chráněný klíč získáte <a href="https://clipsan.com/cs/clipsan/securekey">zde</a>.', 'clipsan-video') ?></span></td>
			</tr>
			<tr valign="top">
				<th scope="row"><label for="clipsan_readall"><?php _e('Načítání videí', 'clipsan-video')?></label></th>
				<td><label><input name="clipsan_readall" type="checkbox"
					<?php echo $clipsan_readall ? 'checked="checked"' : ''?> id="clipsan_readall" value="1"
					/> <?php _e('Načítat nastavení všech videí přímo z Clipsanu', 'clipsan-video')?></label></td>
			</tr>
			<tr valign="top">
				<th scope="row"><label for="clipsan_secureall"><?php _e('Ochrana videí', 'clipsan-video')?></label></th>
				<td><label><input name="clipsan_secureall" type="checkbox"
					<?php echo $clipsan_secureall ? 'checked="checked"' : ''?> id="clipsan_secureall" value="1"
					/> <?php _e('Všechna videa jsou chráněna', 'clipsan-video')?></label></td>
			</tr>
			<tr valign="top">
				<th scope="row"><label for="clipsan_limitedplay"><?php _e('Omezení přehrávání', 'clipsan-video')?></label></th>
				<td><label><input name="clipsan_limitedplay" type="checkbox"
					<?php echo $clipsan_limitedplay ? 'checked="checked"' : ''?> id="clipsan_limitedplay" value="1"
					/> <?php _e('Chráněná videa jsou přehrávána maximálně ze 3 IP adres', 'clipsan-video')?></label></td>
			</tr>
			<?php if (class_exists('PageExpirationRobot')): ?>
			<tr valign="top">
				<th scope="row"><label for="clipsan_per"><?php _e('Page Expiration Robot', 'clipsan-video') ?></label></th>
				<td><select name="clipsan_per" id="clipsan_per">
					<?php foreach (get_posts(array('post_type' => 'per_campaign')) as $post): ?>
						<option value="<?php echo $post->ID ?>"><?php echo $post->post_title?></option>
					<?php endforeach;?>
				</select> <input name="clipsan_per_stage" type="number"
					value="" id="clipsan_per_stage"
					class="regular-text" placeholder="<?php _e('Zadejte ID kroku automatu', 'clipsan-video') ?>"
					 /></td>
			</tr>
			<?php endif ?>
		</table>
		<p class="submit">
			<input type="submit" name="clipsan_submit" id="clipsan_submit"
				class="button-primary" value="<?php _e('Uložit změny', 'clipsan-video')?>" />
		</p>
	</form>
	<?php $init = false; foreach (wp_load_alloptions() as $name => $value):?>
		<?php if (strpos($name, 'clipsan_per_') === 0): ?>
			<?php if (!$init):?><h2><?php _e('Napojení Page Expiration Robot', 'clipsan-video')?></h2><?php $init = true; endif ?>
			<strong><?php $id = substr($name, strlen('clipsan_per_')); $post = get_post($id); echo $post->post_title ?></strong> => [COUNTDOWN id="<?php echo $id ?>"] => Krok automatu ID <?php echo $value ?>
		<?php endif ?>
	<?php endforeach;?>
</div>
<?php
	}
	
	function fixparagraph($content) {
		while ( ($pos = strpos ( $content, '<p>[clip' )) !== false ) {
			$endpos = strpos ( $content, ']' );
			if ($endpos == strpos ( $content, ']</p>' )) {
				$newcontent = substr ( $content, 0, $endpos + 1 );
				$newcontent .= substr ( $content, $endpos + 5 );
				$content = $newcontent;
			} elseif ($endpos == strpos ( $content, ']<br />' )) {
				$newcontent = substr ( $content, 0, $endpos + 1 ) . '<p>';
				$newcontent .= substr ( $content, $endpos + 7 );
				$content = $newcontent;
			}
			$newcontent = substr ( $content, 0, $pos );
			$newcontent .= substr ( $content, $pos + 3 );
			$content = $newcontent;
		}
		return $content;
	}
	
	public function getPlayTableName() {
		global $wpdb;
		return $wpdb->prefix . "clipsan_play";
	}
	public function getPassTableName() {
		global $wpdb;
		return $wpdb->prefix . "clipsan_pass";
	}
}
